@@ -2,7 +2,7 @@
 /*
  *  ssl.h
  *
- *  Copyright (c) 2004 - 2008, clown. All rights reserved.
+ *  Copyright (c) 2004 - 2009, clown. All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions
@@ -36,7 +36,7 @@
  *  The source code is derived from OpenSSL project.
  *  (See the following URL: http://www.openssl.org/source/license.html)
  *
- *  Last-modified: Sat 27 Dec 2008 05:39:00 JST
+ *  Last-modified: Tue 15 Dec 2009 17:59:03 JST
  */
 /* ------------------------------------------------------------------------- */
 #ifndef CLX_SSL_H
@@ -61,10 +61,12 @@
 #include <openssl/err.h>
 #include <openssl/rand.h>
 
+#ifdef CLX_ENABLE_AUTOLINK
 #if defined(_MSC_VER) && (_MSC_VER >= 1200)
 #pragma comment(lib, "libeay32.lib")
 #pragma comment(lib, "ssleay32.lib")
 #endif
+#endif // CLX_ENABLE_AUTOLINK
 
 namespace clx {
 	namespace detail {
@@ -85,7 +87,7 @@ namespace clx {
 		};
 		
 		static ssl_init ssl_init_;
-	}
+	} // namespace detail
 	
 	/* --------------------------------------------------------------------- */
 	//  verify_error
@@ -122,7 +124,8 @@ namespace clx {
 		/* ----------------------------------------------------------------- */
 		//  SSL/TLS version
 		/* ----------------------------------------------------------------- */
-		enum { sslv2 = 0, sslv3, tlsv1, sslv23 };
+		enum { sslv2 = 0x01, sslv3, tlsv1, sslv23 };
+		enum { pem = SSL_FILETYPE_PEM, asn1 = SSL_FILETYPE_ASN1 };
 		
 		template <int Version, int Family> class basic_acceptor;
 		/* ----------------------------------------------------------------- */
@@ -133,98 +136,205 @@ namespace clx {
 			int Family
 		>
 		class basic_socket : public basic_rawsocket<SOCK_STREAM, Family, 0> {
-			friend class basic_acceptor<Version, Family>;
-			
+		private:
+			typedef basic_rawsocket<SOCK_STREAM, Family, 0> super;
 		public:
-			typedef basic_rawsocket<SOCK_STREAM, Family, 0> rawsocket;
 			typedef SSL handler;
-			typedef basic_sockaddress<Family, IPPROTO_TCP> sockaddress;
+			typedef basic_sockaddress<Family, IPPROTO_TCP> address_type;
 			typedef char char_type;
 			typedef typename std::basic_string<char> string_type;
 			
-			// constructor and destructor
-			basic_socket() : rawsocket(), p_(), addr_() {}
+			/* ----------------------------------------------------------------- */
+			//  constructor
+			/* ----------------------------------------------------------------- */
+			basic_socket() : super(), p_(), addr_(), connect_(false) {}
 			
-			basic_socket(const basic_socket& cp) : rawsocket(cp), p_(cp.p_), addr_(cp.addr_) {}
+			/* ----------------------------------------------------------------- */
+			//  constructor
+			/* ----------------------------------------------------------------- */
+			basic_socket(const basic_socket& cp) :
+				super(cp), p_(cp.p_), addr_(cp.addr_), connect_(cp.connect_) {}
 			
+			/* ----------------------------------------------------------------- */
+			//  constructor
+			/* ----------------------------------------------------------------- */
 			template <class Socket>
-			basic_socket(const Socket& cp) :
-				rawsocket(cp), p_(), addr_(cp.address()) {
-				this->xinit(client);
-				p_->connect() = cp.is_connect();
-				this->xconnect();
+			explicit basic_socket(const Socket& cp) :
+				super(cp), p_(), addr_(cp.address()), connect_(cp.is_connect()) {
+				this->init();
 			}
 			
-			// for client ssl socket
-			explicit basic_socket(const char_type* host, int port) :
-				rawsocket(), p_(), addr_(host, port) {
-				this->xinit(client);
-				this->xconnect();
+			/* ----------------------------------------------------------------- */
+			/*
+			 *  constructor
+			 *
+			 *  The constructor is called when making a new client ssl socket.
+			 */
+			/* ----------------------------------------------------------------- */
+			basic_socket(const char_type* host, int port) :
+				super(), p_(), addr_(host, port) {
+				this->connect_and_handshake();
 			}
 			
-			explicit basic_socket(const string_type& host, int port) :
-				rawsocket(), p_(), addr_(host, port) {
-				this->xinit(client);
-				this->xconnect();
+			/* ----------------------------------------------------------------- */
+			/*
+			 *  constructor
+			 *
+			 *  The constructor is called when making a new client ssl socket.
+			 */
+			/* ----------------------------------------------------------------- */
+			basic_socket(const string_type& host, int port) :
+				super(), p_(), addr_(host, port) {
+				this->connect_and_handshake();
 			}
 			
-			// for server ssl socket
-			explicit basic_socket(socket_int s, const sockaddress& addr) :
-				rawsocket(s), p_(), addr_(addr) {
-				this->xinit(server);
-				p_->connect() = true;
+			/* ----------------------------------------------------------------- */
+			/*
+			 *  constructor
+			 *
+			 *  The constructor is called when making a new server ssl socket.
+			 */
+			/* ----------------------------------------------------------------- */
+			basic_socket(socket_int s, const address_type& addr) :
+				super(s), p_(), addr_(addr) {
+				this->init();
+				connect_ = true;
 			}
 			
+			/* ----------------------------------------------------------------- */
+			//  destructor
+			/* ----------------------------------------------------------------- */
 			virtual ~basic_socket() throw() {}
 			
+			/* ----------------------------------------------------------------- */
+			//  assignment operator.
+			/* ----------------------------------------------------------------- */
 			basic_socket& operator=(const basic_socket& cp) {
 				addr_ = cp.addr_;
-				this->assign(static_cast<rawsocket>(cp));
+				this->assign(static_cast<super>(cp));
 				p_ = cp.p_;
 				return *this;
 			}
 			
-			// for client ssl socket
+			/* ----------------------------------------------------------------- */
+			//  connect
+			/* ----------------------------------------------------------------- */
 			basic_socket& connect(const char_type* host, int port) {
 				addr_.assign(host, port);
-				return this->xconnect();
+				return this->connect_and_handshake();
 			}
 			
+			/* ----------------------------------------------------------------- */
+			//  connect
+			/* ----------------------------------------------------------------- */
 			basic_socket& connect(const string_type& host, int port) {
 				return this->connect(host.c_str(), port);
 			}
 			
+			/* ----------------------------------------------------------------- */
+			//  handshake
+			/* ----------------------------------------------------------------- */
+			basic_socket& handshake() {
+				if (this->is_handshake()) return *this;
+				this->create_ssl(client);
+				
+				RAND_poll();
+				mt19937 engine(static_cast<unsigned long>(std::time(NULL)));
+				random<unsigned short> rng(engine);
+				while (RAND_status() == 0) {
+					unsigned short value = rng() % 65536;
+					RAND_seed(&value, sizeof(value));
+				}
+				
+				char err[1024];
+				if (SSL_connect(p_->ssl()) <= 0) {
+					std::memset(err, '\0', sizeof(err));
+					ERR_error_string(ERR_get_error(), err);
+					throw std::runtime_error(err);
+				}
+				p_->handshake() = true;
+				
+				if (p_->verify()) {
+					int status = SSL_get_verify_result(p_->ssl());
+					if (status != X509_V_OK) throw verify_error(status);
+				}
+				
+				return *this;
+			}
+			
+			/* ----------------------------------------------------------------- */
+			//  handshake
+			/* ----------------------------------------------------------------- */
+			basic_socket& handshake(const char_type* cert, const char_type* priv, int type = pem) {
+				if (this->is_handshake()) return *this;
+				this->create_ssl(server);
+				
+				SSL_use_certificate_file(p_->ssl(), cert, type);
+				SSL_use_PrivateKey_file(p_->ssl(), priv, type);
+				
+				char err[1024];
+				if (SSL_accept(p_->ssl()) <= 0) {
+					std::memset(err, '\0', sizeof(err));
+					ERR_error_string(ERR_get_error(), err);
+					throw std::runtime_error(err);
+				}
+				p_->handshake() = true;
+				
+				return *this;
+			}
+			
+			/* ----------------------------------------------------------------- */
+			//  send
+			/* ----------------------------------------------------------------- */
 			virtual int send(const char_type* src, int n) {
-				if (p_.get() == NULL || p_->ssl() == NULL) return -1;
+				if (!p_ || !p_->ssl()) return -1;
 				return SSL_write(p_->ssl(), src, n);
 			}
 			
+			/* ----------------------------------------------------------------- */
+			//  send
+			/* ----------------------------------------------------------------- */
 			virtual int send(const string_type& src) {
 				return this->send(src.c_str(), static_cast<int>(src.size()));
 			}
 			
+			/* ----------------------------------------------------------------- */
+			//  recv
+			/* ----------------------------------------------------------------- */
 			virtual int recv(char_type* dest, int n) {
-				if (p_.get() == NULL || p_->ssl() == NULL) return -1;
+				if (!p_ || !p_->ssl()) return -1;
 				return SSL_read(p_->ssl(), dest, n);
 			}
 			
+			/* ----------------------------------------------------------------- */
+			//  close
+			/* ----------------------------------------------------------------- */
 			virtual void close() {
-				rawsocket::close();
-				if (p_.get() != NULL) p_->close();
+				super::close();
+				if (p_) p_->close();
+				connect_ = false;
 			}
 			
+			/* ----------------------------------------------------------------- */
+			//  pending
+			/* ----------------------------------------------------------------- */
 			virtual int pending() const {
-				if (p_.get() == NULL || p_->ssl() == NULL) return 0;
+				if (!p_ || !p_->ssl()) return 0;
 				return SSL_pending(p_->ssl());
 			}
 			
+			/* ----------------------------------------------------------------- */
+			//  verify_locations
+			/* ----------------------------------------------------------------- */
 			bool verify_locations(const char_type* file, const char_type* dir = NULL) {
-				if (p_.get() == NULL) this->xinit(client);
-				if (SSL_CTX_load_verify_locations(p_->ctx(), file, dir) == 0) return false;
+				if (!p_ || !p_->ctx() || SSL_CTX_load_verify_locations(p_->ctx(), file, dir) == 0) return false;
 				p_->verify() = true;
 				return true;
 			}
 			
+			/* ----------------------------------------------------------------- */
+			//  verify_locations
+			/* ----------------------------------------------------------------- */
 			bool verify_locations(const string_type& file, const string_type& dir = string_type()) {
 				if (file.empty() && dir.empty()) return false;
 				else if (dir.empty()) return this->verify_locations(file.c_str(), NULL);
@@ -232,27 +342,29 @@ namespace clx {
 				else return this->verify_locations(file.c_str(), dir.c_str());
 			}
 			
-			bool is_connect() const { return (p_.get() != NULL) ? p_->connect() && p_->handshake() : false; }
-			const sockaddress& address() const { return addr_; }
-			handler* ssl() { return (p_.get() != NULL) ? p_->ssl() : NULL; }
-			const handler* ssl() const { return (p_.get() != NULL) ? p_->ssl() : NULL; }
+			/* ----------------------------------------------------------------- */
+			//  access methods.
+			/* ----------------------------------------------------------------- */
+			bool is_connect() const { return connect_; }
+			bool is_handshake() const { return p_ ? p_->handshake() : false; }
+			const address_type& address() const { return addr_; }
+			handler* ssl() { return p_ ? p_->ssl() : NULL; }
+			const handler* ssl() const { return p_ ? p_->ssl() : NULL; }
 			
 		private:
 			typedef SSL_CTX ctx_type;
-			
-			enum { client = 0, server = 1};
+			enum { client = 0x10, server = 0x20 };
 			
 			class ssl_impl {
 			public:
 				typedef ctx_type* ctx_pointer;
 				typedef handler* handle_pointer;
 				
-				ssl_impl() : ctx_(NULL), ssl_(NULL), connect_(false), handshake_(false), verify_(false) {}
+				ssl_impl() : ctx_(NULL), ssl_(NULL), handshake_(false), verify_(false) {}
 				~ssl_impl() { this->close(); }
 				
 				ctx_pointer& ctx() { return ctx_; }
 				handle_pointer& ssl() { return ssl_; }
-				bool& connect() { return connect_; }
 				bool& handshake() { return handshake_; }
 				bool& verify() { return verify_; }
 				
@@ -268,7 +380,6 @@ namespace clx {
 						ctx_ = NULL;
 					}
 					
-					connect_ = false;
 					handshake_ = false;
 					verify_ = false;
 				}
@@ -276,19 +387,22 @@ namespace clx {
 			private:
 				ctx_pointer ctx_;
 				handle_pointer ssl_;
-				bool connect_;
 				bool handshake_;
 				bool verify_;
 			};
 			
 			shared_ptr<ssl_impl> p_;
-			sockaddress addr_;
+			address_type addr_;
+			bool connect_;
 			
-			basic_socket& xinit(int mode) {
+			basic_socket& init() {
 				if (!this->is_open()) this->open();
 				if (!this->is_open()) throw socket_error("open");
-				if (p_.get() == NULL) p_ = shared_ptr<ssl_impl>(new ssl_impl());
-				
+				if (!p_) p_ = shared_ptr<ssl_impl>(new ssl_impl());
+				return *this;
+			}
+			
+			basic_socket& create_ssl(int mode) {
 				SSL_METHOD *(*f)() = NULL;
 				switch (Version) {
 				case sslv2:
@@ -310,12 +424,12 @@ namespace clx {
 					break;
 				}
 				
-				if (p_->ctx() == NULL) p_->ctx() = SSL_CTX_new(f());
-				if (p_->ctx() == NULL) throw std::runtime_error("SSL_CTX_new failed");
+				if (!p_->ctx()) p_->ctx() = SSL_CTX_new(f());
+				if (!p_->ctx()) throw std::runtime_error("SSL_CTX_new failed");
 				
 				char err[1024];
-				if (p_->ssl() == NULL) p_->ssl() = SSL_new(p_->ctx());
-				if (p_->ssl() == NULL || SSL_set_fd(p_->ssl(), this->socket()) == 0) {
+				if (!p_->ssl()) p_->ssl() = SSL_new(p_->ctx());
+				if (!p_->ssl() || SSL_set_fd(p_->ssl(), this->socket()) == 0) {
 					std::memset(err, '\0', sizeof(err));
 					ERR_error_string(ERR_get_error(), err);
 					throw std::runtime_error(err);
@@ -325,40 +439,15 @@ namespace clx {
 			}
 			
 			// for client ssl socket
-			basic_socket& xconnect() {
-				if (p_.get() == NULL) this->xinit(client);
-				
-				if (!p_->connect()) {
+			basic_socket& connect_and_handshake() {
+				this->init();
+				if (!this->is_connect()) {
 					if (::connect(this->socket(), (struct sockaddr*)addr_.data(), addr_.size()) == -1) {
 						throw clx::socket_error("connect");
 					}
-					p_->connect() = true;
+					connect_ = true;
 				}
-				
-				if (!p_->handshake()) {
-					RAND_poll();
-					mt19937 engine(static_cast<unsigned long>(std::time(NULL)));
-					random<unsigned short> rng(engine);
-					while (RAND_status() == 0) {
-						unsigned short value = rng() % 65536;
-						RAND_seed(&value, sizeof(value));
-					}
-					
-					char err[1024];
-					if (SSL_connect(p_->ssl()) <= 0) {
-						std::memset(err, '\0', sizeof(err));
-						ERR_error_string(ERR_get_error(), err);
-						throw std::runtime_error(err);
-					}
-					p_->handshake() = true;
-				}
-				
-				if (p_->verify()) {
-					int status = SSL_get_verify_result(p_->ssl());
-					if (status != X509_V_OK) throw verify_error(status);
-				}
-				
-				return *this;
+				return this->handshake();
 			}
 		};
 		
@@ -377,21 +466,21 @@ namespace clx {
 			int Family
 		>
 		class basic_acceptor : public basic_rawsocket<SOCK_STREAM, Family> {
+		private:
+			typedef basic_rawsocket<SOCK_STREAM, Family> super;
 		public:
-			typedef basic_rawsocket<SOCK_STREAM, Family> rawsocket;
-			typedef basic_sockaddress<Family, IPPROTO_TCP> sockaddress;
+			typedef basic_sockaddress<Family, IPPROTO_TCP> address_type;
 			typedef char char_type;
 			typedef typename std::basic_string<char> string_type;
 			
-			enum { pem = SSL_FILETYPE_PEM, asn1 = SSL_FILETYPE_ASN1 };
 			enum { nmax = 100 };
 			
 			// constructor and destructor
-			basic_acceptor() : rawsocket(), nlisten_(nmax) {}
+			basic_acceptor() : super(), nlisten_(nmax) {}
 			
 			explicit basic_acceptor(int port, int n = nmax) :
-				rawsocket(), addr_(port), nlisten_(nmax) {
-				this->xopen();
+				super(), addr_(port), nlisten_(nmax) {
+				this->start();
 			}
 			
 			virtual ~basic_acceptor() throw() {}
@@ -400,14 +489,13 @@ namespace clx {
 			basic_acceptor& open(int port, int n = nmax) {
 				addr_.assign(port);
 				nlisten_ = n;
-				return this->xopen();
+				return this->start();
 			}
 			
-			basic_socket<Version, Family> accept(const char_type* cert, const char_type* priv,
-				int type = pem) {
+			basic_socket<Version, Family> accept(const char_type* cert, const char_type* priv, int type = pem) {
 				socket_int s = -1;
-				sockaddress addr;
-				typename sockaddress::inet_type tmp;
+				address_type addr;
+				typename address_type::inet_type tmp;
 				socklen_type len = sizeof(tmp);
 				
 				for (int foo = 0; foo < 10; ++foo) {
@@ -417,34 +505,23 @@ namespace clx {
 				}
 				if (s == -1) throw socket_error("accept");
 				
-				addr = tmp;
-				
+				addr.assign(tmp);
 				basic_socket<Version, Family> client(s, addr);
-				SSL_use_certificate_file(client.ssl(), cert, type);
-				SSL_use_PrivateKey_file(client.ssl(), priv, type);
-				
-				char err[1024];
-				if (SSL_accept(client.ssl()) <= 0) {
-					std::memset(err, '\0', sizeof(err));
-					ERR_error_string(ERR_get_error(), err);
-					throw std::runtime_error(err);
-				}
-				client.p_->handshake() = true;
+				client.handshake(cert, priv, pem);
 				
 				return client;
 			}
 			
-			basic_socket<Version, Family> accept(const string_type& cert, const string_type& priv,
-				int type = pem) {
+			basic_socket<Version, Family> accept(const string_type& cert, const string_type& priv, int type = pem) {
 				return this->accept(cert.c_str(), priv.c_str(), type);
 			}
 			
 		private:
-			sockaddress addr_;
+			address_type addr_;
 			int nlisten_;
 			
-			basic_acceptor& xopen() {
-				if (rawsocket::open() < 0) throw socket_error("socket");
+			basic_acceptor& start() {
+				if (super::open() < 0) throw socket_error("socket");
 				if (::bind(this->socket(), (struct sockaddr*)addr_.data(), addr_.size()) == -1) {
 					throw socket_error("bind");
 				}
